#include <algorithm>
#include <array>
#include <cstdint>
#include <iostream>
#include <string>
#include <vector>

// Use a dynamic vector for the children, and iterate over
// all children to find next node.
class Trie {
public:
    Trie(): children_(1), hits_(1, 0) {}

    uint32_t follow(uint32_t node, uint32_t x) const {
        for (auto [idx, val] : children_[node]) {
            if (val == x) {
                return idx;
            }
        }

        return 0;
    }

    uint32_t follow_create(uint32_t node, uint32_t x) {
        for (auto [idx, val] : children_[node]) {
            if (val == x) {
                return idx;
            }
        }

        uint32_t next_node = children_.size();
        children_[node].emplace_back(next_node, x);
        children_.emplace_back();
        hits_.push_back(0);

        return next_node;
    }

    void insert(const std::vector<int32_t>& data) {
        uint32_t node = 0;
        for (int32_t x : data) {
            ++hits_[node];

            node = follow_create(node, x);
        }

        ++hits_[node];
    }

    int32_t query(const std::vector<int32_t>& data) const {
        uint32_t node = 0;
        for (int32_t x : data) {
            node = follow(node, x);
            if (!node) {
                return 0;
            }
        }

        return hits_[node];
    }
private:
    std::vector<std::vector<std::pair<uint32_t, uint32_t>>> children_;
    std::vector<int32_t> hits_;
};

std::vector<int32_t> make_one(std::string_view s) {
    int32_t n = s.size();
    std::vector<int32_t> keys;
    keys.reserve(n);
    for (int32_t i = 0; i < n; ++i) {
        keys.push_back(s[i] - 'a');
    }

    return keys;
}

std::vector<int32_t> make_both(std::string_view s, std::string_view t) {
    int32_t n = s.size();
    std::vector<int32_t> keys;
    keys.reserve(n);
    for (int32_t i = 0; i < n; ++i) {
        keys.push_back(26 * (s[i] - 'a') + (t[n - 1 - i] - 'a'));
    }

    return keys;
}

int main() {
    int32_t n, q;
    std::cin >> n >> q;

    Trie prefix, suffix;
    Trie and_trie;
    for (int32_t word_idx = 0; word_idx < n; ++word_idx) {
        std::string word;
        std::cin >> word;

        std::vector<int32_t> forward = make_one(word);
        prefix.insert(forward);

        std::vector<int32_t> backwards(forward.begin(), forward.end());
        std::reverse(backwards.begin(), backwards.end());
        suffix.insert(backwards);

        std::vector<int32_t> both = make_both(word, word);
        and_trie.insert(both);
    }

    for (int32_t i = 0; i < q; ++i) {
        std::string op, l, r;
        std::cin >> op >> l >> r;

        std::vector<int32_t> and_query = make_both(l, r);
        int32_t and_count = and_trie.query(and_query);

        int32_t ans;
        if (op == "AND") {
            ans = and_count;
        } else {
            int32_t pref_count = prefix.query(make_one(l));
            std::vector<int32_t> backwards = make_one(r);
            std::reverse(backwards.begin(), backwards.end());
            int32_t suff_count = suffix.query(backwards);

            if (op == "OR") {
                ans = pref_count + suff_count - and_count;
            } else {
                ans = pref_count + suff_count - 2 * and_count;
            }
        }

        std::cout << ans << '\n';
    }
}
